# -*- Mode: Python; test-case-name: codeina.test.test_httpdownload -*-
# vi:si:et:sw=4:sts=4:ts=4

import os

from codeina.test.common import unittest
from codeina import check 

class TestVersion(unittest.TestCase):

    def test_init(self):
        try:
              d = check.Version("")
        except ValueError:
            pass
        else:
            self.fail('expected a ValueError exception')

    def test_is_newer_than(self):
        d = check.Version("0.0.1.1")

        self.assertEqual(True,  d.is_newer_than(check.Version("0.0.1.0")))
        self.assertEqual(True,  d.is_newer_than(check.Version("0.0.1")))

        self.assertEqual(False, d.is_newer_than(check.Version("0.0.1.1")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.0.1.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.0.2.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.1.2.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("1.1.2.2")))

        self.assertEqual(False, d.is_newer_than(check.Version("0.0.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.1.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("1.1.2")))

        d = check.Version("0.10.1")

        self.assertEqual(False,  d.is_newer_than(check.Version("0.10.1.0")))
        self.assertEqual(False,  d.is_newer_than(check.Version("0.10.1.1")))
        self.assertEqual(True,  d.is_newer_than(check.Version("0.10.0.1")))

        self.assertEqual(False, d.is_newer_than(check.Version("0.10.1.1")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.10.1.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.10.2.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.11.2.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("1.11.2.2")))

        self.assertEqual(False, d.is_newer_than(check.Version("0.10.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("0.11.2")))
        self.assertEqual(False, d.is_newer_than(check.Version("1.11.2")))

        self.assertEqual(True, check.Version("0.10.11").is_newer_than(check.Version("0.10.9")))

        d = check.Version("0.1.13.1")

        self.assertEqual(False, d.is_newer_than(check.Version("0.1.13.1")))
        self.assertEqual(True,  d.is_newer_than(check.Version("0.0.0.2")))
        self.assertEqual(True,  d.is_newer_than(check.Version("0.0.13.2")))
        self.assertEqual(True,  d.is_newer_than(check.Version("0.1.13.0")))
        self.assertEqual(False, d.is_newer_than(check.Version("1.0.13.2")))
        #self.assertEqual(False, d.is_newer_than(check.Version("SVN-r600")))

    def test_is_newer_than_svn_revision(self):
        d = check.Version("SVN-r500")
        #self.assertEqual(False, d.is_newer_than(check.Version("SVN-r600")))
        #self.assertEqual(True,  d.is_newer_than(check.Version("SVN-r400")))


    def test_fulfils_requirement(self):
        d = check.Version("0.10.3.1")

        self.assertEqual(False, d.fulfils_requirement(check.Version("0.10.3.0")))
        self.assertEqual(True,  d.fulfils_requirement(check.Version("0.10.3.1")))
        self.assertEqual(True,  d.fulfils_requirement(check.Version("0.10.3.2")))

    def test_get_string(self):
        version = "0.1.13.1"

        d = check.Version(version)
        self.assertEqual(version, str(d))
        self.assertEqual(version, d.__str__())

class Test_gst_plugin_version(unittest.TestCase):
    
    def test_get_plugin_version_unknown(self):
        version = check.get_plugin_version("unknown_plugin")
        self.assertEqual(None, version)

    def test_get_plugin_version(self):
        version = check.get_plugin_version("audioconvert")
        self.assertEqual(True, version.is_newer_than(check.Version("0.0.0.1") ) )
        self.assertEqual(True, version.is_newer_than(check.Version("0.10.0") ) )

class Test_gst_plugin_from_element_factory_name(unittest.TestCase):
    
    def test_get_plugin_from_element_factory_name_unknown(self):
        plugin = check.get_plugin_from_element_factory_name("unknown_plugin")
        self.assertEqual(None, plugin)

    def test_get_plugin_from_element_factory_name(self):
        plugin = check.get_plugin_from_element_factory_name("audioconvert")
        self.assertEqual(True, str(plugin).endswith("(gst.Plugin)"))

class Test_gst_plugin_mtime(unittest.TestCase):
    
    def test_get_plugin_mtime_unknown(self):
        plugin = check.get_plugin_from_element_factory_name("unknown_plugin")
        plugin_mtime = check.get_plugin_mtime( plugin )
        self.assertEqual(None, plugin_mtime)

    def test_get_plugin_mtime(self):
        """ check the returned mtime is in the gmtime format """
        from time import gmtime
        plugin = check.get_plugin_from_element_factory_name("audioconvert")
        self.assertNotEqual( plugin, None, "Could not find audioconvert!" + \
            "do you have gstreamer0.10-plugins-base installed?" )
        plugin_mtime = check.get_plugin_mtime( plugin )
        self.assertEqual( len(gmtime()), len(plugin_mtime) )

class Test_gst_element_mtime(unittest.TestCase):
    
    def test_get_element_mtime_unknown(self):
        plugin_mtime = check.get_element_mtime( "unknown_plugin" )
        self.assertEqual(None, plugin_mtime)

    def test_get_plugin_mtime(self):
        """ check the returned mtime is in the gmtime format """
        from time import gmtime
        plugin_mtime = check.get_element_mtime("audioconvert")
        self.assertEqual( len( gmtime() ) , len(plugin_mtime) )

class Test_gst_element_version(unittest.TestCase):
    
    def test_get_element_version_unknown(self):
        version = check.get_element_version("unknown_plugin")
        self.assertEqual( None, version )

    def test_get_element_version_unknown(self):
        version = check.get_element_version("audioconvert")
        self.assertNotEqual( None, version )

class Test_find_system_details(unittest.TestCase):
    
    def test_find_system_details(self):
        check.find_system_details()

class Test_initial_checks(unittest.TestCase):
    
    def test_initial_checks(self):
        check.initial_checks()

