# vi:si:et:sw=4:sts=4:ts=4

import os

from codeina.test.common import unittest
from codeina import productfile

class TestProductFile(unittest.TestCase):

    def setUp (self):
        self.pf = productfile.productFile ()
        self.pf.set_filename ('/tmp/example.codeina')

    def test_set_filename (self):
        self.assertEquals (self.pf.get_filename (), '/tmp/example.codeina')

    def test_add_product (self):
        self.assertEquals (self.pf.product_count (), 0)
        self.assertEquals (self.pf.product_add ('fluendo-mp3.tar.bz2'), True)
        self.assertEquals (self.pf.product_count (), 1)

    def test_add_product_twice (self):
        self.pf.product_add ('fluendo-mp3.tar.bz2')
        self.assertEquals (self.pf.product_add ('fluendo-mp3.tar.bz2'), False)
        self.assertEquals (self.pf.product_count (), 1)

    def test_save_file (self):
        self.pf.product_add ('fluendo-mp3.tar.bz2')
        self.pf.product_add ('fluendo-mp4.tar.bz2')
        self.pf.save ()
        lines = open('/tmp/example.codeina', 'r').readlines ()
        self.assertEquals (len (lines), 2)
        self.assertEquals (lines[0], 'fluendo-mp3.tar.bz2\n')
        self.assertEquals (lines[1], 'fluendo-mp4.tar.bz2\n')

    def test_load_file (self):
        self.pf.product_add ('fluendo-mp3.tar.bz2')
        self.pf.product_add ('fluendo-mp4.tar.bz2')
        self.pf.save ()
        pf2 = productfile.productFile ()
        pf2.set_filename ('/tmp/example.codeina')
        pf2.load ()
        self.assertEquals (pf2.product_add ('fluendo-mp3.tar.bz2'), False)
        self.assertEquals (pf2.product_add ('fluendo-mp4.tar.bz2'), False)
        self.assertEquals (pf2.product_count (), 2)

    def test_add_product_and_save (self):
        if os.path.exists ('/tmp/example.codeina'):
            os.system ('rm -rf /tmp/example.codeina')
        self.assertEquals (os.path.exists ('/tmp/example.codeina'), False)
        self.assertEquals (self.pf.product_quick_add ('fluendo-mp5.tar.bz2'), True)
        self.assertEquals (self.pf.product_quick_add ('fluendo-mp5.tar.bz2'), False)
        pf2 = productfile.productFile ()
        pf2.set_filename ('/tmp/example.codeina')
        pf2.load ()
        self.assertEquals (pf2.product_add ('fluendo-mp5.tar.bz2'), False)
        self.assertEquals (pf2.product_count (), 1)
