# -*- Mode: Python; test-case-name: codeina.test.test_product_list -*-
# vi:si:et:sw=4:sts=4:ts=4

import os
import StringIO

from codeina.test.common import unittest
from codeina import productlistparser
from codeina import productlist
from codeina import check

class TestProduct(unittest.TestCase):
    def setUp(self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='gutsy' arch='i586' distro='ubuntu'/>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.ppc.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='ppc' distro='ubuntu'/>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.ppc.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='i686' distro='generic'/>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        self.assertEquals(len(productlist.products), 1)
        self.p = productlist.products[0]

    def test_flavor_from_xml(self):
        self.assertEquals (len (self.p.flavors), 3)
        fl_i586 = self.p.flavors[0]
        fl_ppc = self.p.flavors[1]
        fl_i686 = self.p.flavors[2]
        self.assertEquals (fl_ppc.arch, 'ppc')
        self.assertEquals (fl_i586.arch, 'i586')
        self.assertEquals (fl_i586.get_distro (), 'ubuntu')
        self.assertEquals (fl_i686.arch, 'i686')

    def test_flavor_distro(self):
        fl_i586 = self.p.flavors[0]
        self.assertEquals (fl_i586.get_distro (), 'ubuntu')
        self.assertEquals (fl_i586.is_distro('ubuntu'), True)
        self.assertEquals (fl_i586.is_compatible_distro(['ubuntu', 'gutsy']), True)

    def test_flavor_distro_compatible_generic (self):
        fl_i686 = self.p.flavors[2]
        self.assertEquals (fl_i686.get_distro (), 'generic')
        self.assertEquals (fl_i686.is_distro('generic'), True)
        self.assertEquals (fl_i686.is_compatible_distro(['generic','any']), True)
        self.assertEquals (fl_i686.is_compatible_distro(['ubuntu','any']), True)
        # Every distro is compatible with generic products
        self.assertEquals (fl_i686.is_compatible_distro(['MyOwnDistro','first']), True)

    def test_product_for_flavors (self):
        distro_list = self.p.get_flavor_distros ()
        self.assertEquals (distro_list, ['ubuntu', 'generic'])

    def test_product_distro_version (self):
        fl_i586 = self.p.flavors[0]
        self.assertEquals (fl_i586.get_distro_version (), 'gutsy')
        self.assertEquals (fl_i586.is_compatible_distro (['ubuntu', 'gutsy']), True)
        self.assertEquals (fl_i586.is_compatible_distro (['ubuntu', 'dapper']), False)

    def test_product_get_best_flavor_for_current_system (self):
        pass

    def test_flavor_is_compatible_distro_ubuntu_any (self):
        fl_ppc = self.p.flavors[1]
        self.assertEquals (fl_ppc.get_distro (), 'ubuntu')
        self.assertEquals (fl_ppc.get_distro_version (), 'any')
        self.assertEquals (fl_ppc.is_compatible_distro (['ubuntu', 'myfirst']), True)
        self.assertEquals (fl_ppc.is_compatible_distro (['mandriva', 'any']), False)

    def test_is_compatible (self):
        fl_ppc = self.p.flavors[1]
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', 'ubuntu', check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', 'ubuntu', check.Version('0.10.0'), check.Version('0.10.5')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', 'ubuntu', check.Version('1.14.4'), check.Version('0.10.5')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'i386', 'generic', check.Version('0.10.0'), check.Version('0.10.0')), False)

    def test_is_compatible_version_any (self):
        fl_ppc = self.p.flavors[1]
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['ubuntu', 'any'], check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['ubuntu', 'dapper'], check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['ubuntu', 'gutsy'], check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['generic', 'any'], check.Version('0.10.0'), check.Version('0.10.0')), False)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['generic', 'dapper'], check.Version('0.10.0'), check.Version('0.10.0')), False)
        self.assertEquals (fl_ppc.is_compatible ('Linux', 'ppc', ['generic', 'gutsy'], check.Version('0.10.0'), check.Version('0.10.0')), False)

    def test_is_compatible_ix86 (self):
        fl_i586 = self.p.flavors[0]
        fl_i686 = self.p.flavors[2]
        self.assertEquals (fl_i586.is_compatible ('Linux', 'ppc', 'generic', check.Version('0.10.0'), check.Version('0.10.0')), False)
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i686', 'generic', check.Version('0.10.0'), check.Version('0.10.0')), False)
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i586', 'ubuntu', check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i386', 'ubuntu', check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_i686.is_compatible ('Linux', 'i686', 'generic', check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_i686.is_compatible ('Linux', 'i586', 'generic', check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_i686.is_compatible ('Linux', 'i386', 'generic', '0.10.0', '0.10.0'), True)

    def test_is_compatible_version_gutsy (self):
        fl_i586 = self.p.flavors[0]
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i586', ['ubuntu', 'gutsy'], check.Version('0.10.0'), check.Version('0.10.0')), True)
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i586', ['ubuntu', 'dapper'], check.Version('0.10.0'), check.Version('0.10.0')), False)
        self.assertEquals (fl_i586.is_compatible ('Linux', 'i586', ['ubuntu', 'any'], check.Version('0.10.0'), check.Version('0.10.0')), False)
