# vi:si:et:sw=4:sts=4:ts=4

import os
import StringIO

from codeina.test.common import unittest
from codeina import productlistparser, productlist, configure

class TestProductList(unittest.TestCase):
    def setUp(self):
        self.xml_data = open(configure.make_xml_path('fluendo.xml'))
        self.parser = productlistparser.ProductListParser()
        self.productlist = self.parser.parse(self.xml_data, "fluendo_3")

    def test_fluendo_general_inspection (self):
        self.assertEquals (len(self.productlist.products), 10)
        self.assertEquals (self.productlist.products[0].nick, 'wma-bundle')
        self.assertEquals (len(self.productlist.products[0].plugins), 2)
        self.assertEquals (len(self.productlist.products[0].plugins[0].elements), 2)
        self.assertEquals (self.productlist.products[0].plugins[0].elements[0].name, 'fluwmadec')

    def testProductListCountSelected(self):
        self.productlist.products[0].selected = True 
        self.assertEquals (self.productlist.count_selected(), 1)

    def testProductListGetSelected(self):
        p = self.productlist.products[0]

        p.selected = True 
        self.assertEquals(True, p.get_selected())

        p.set_selected(False)
        self.assertEquals(False, p.get_selected())

        p.set_selected(True)
        self.assertEquals(True, p.selected)

        # make sure we get the correct oject
        selected_product = self.productlist.get_selected()[0]
        self.assertEquals(selected_product, p)
        # make sure it's not a copy of it 
        selected_product.selected = False
        self.assertEquals(p, selected_product)

    def test_product_has_for_this_system (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='i386' distro='generic'/>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.ppc.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='ppc' distro='generic'/>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        from codeina import check
        check.OS_NAME = 'Linux'
        check.ARCH_NAME = 'i386'
        check.DISTRO_NAME = 'ubuntu'
        check.DISTRO_VERSION = 'gutsy'
        check.GST_CORE_VERSION = check.Version ('0.10.6')
        check.GST_PLUGINS_BASE_VERSION = check.Version ('0.10.6')
        self.assertEquals (productlist.has_products_for_current_system (), True)

    def test_product_has_no_flavor_for_system (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='exotic-i386' distro='generic'/>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        from codeina import check
        check.OS_NAME = 'Linux'
        check.ARCH_NAME = 'i386'
        check.DISTRO_NAME = 'ubuntu'
        check.DISTRO_VERSION = 'gutsy'
        check.GST_CORE_VERSION = check.Version ('0.10.6')
        check.GST_PLUGINS_BASE_VERSION = check.Version ('0.10.6')
        self.assertEquals (productlist.has_products_for_current_system (), False)

    def test_product_list_filter_for_system (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='p1' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='exotic-i386' distro='generic'/>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='p2' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='i386' distro='generic'/>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        from codeina import check
        check.OS_NAME = 'Linux'
        check.ARCH_NAME = 'i386'
        check.DISTRO_NAME = 'ubuntu'
        check.DISTRO_VERSION = 'gutsy'
        check.GST_CORE_VERSION = check.Version ('0.10.6')
        check.GST_PLUGINS_BASE_VERSION = check.Version ('0.10.6')
        self.assertEquals (productlist.has_products_for_current_system (), True)
        self.assertEquals (productlist.filter_products_for_current_system (), True)
        self.assertEquals ( len (productlist.products), 1)

    def test_product_list_get_intersection (self):
        xml = """
<products id='fluendo' url='http://fluendo.com'>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='p1' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='exotic-i386' distro='generic'/>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='p2' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='i386' distro='generic'/>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        self.assertEquals (productlist.product_length (), 2)
        product_1 = productlist.products[0]
        list2 = productlist.get_intersection ([product_1])
        self.assertEquals (list2.product_length (), 1)
        self.assertEquals (list2.id, 'fluendo')
        self.assertEquals (list2.url, 'http://fluendo.com')
        self.assertEquals (list2.products[0], product_1)
        product_2 = productlist.products[0]
        list3 = productlist.get_intersection ([product_2])
        self.assertEquals (list3.product_length (), 1)
        self.assertEquals (list3.id, 'fluendo')
        self.assertEquals (list3.url, 'http://fluendo.com')
        self.assertEquals (list3.products[0], product_2)

class TestProductUbuntuFluendo(unittest.TestCase):

    """ some scenari to match ubuntu vs. fluendo packages """

    def setUp(self):
        self.xml_data = open(configure.make_xml_path('ubuntu_7.10.xml'))
        self.parser = productlistparser.ProductListParser()
        self.ubuntu = self.parser.parse(self.xml_data, "ubuntu_gutsy").products

        self.xml_data = open(configure.make_xml_path('fluendo.xml'))
        self.parser = productlistparser.ProductListParser()
        self.fluendo = self.parser.parse(self.xml_data, id= 'fluendo_3').products

        self.products = self.fluendo+self.ubuntu

    def testLoadProductFiles(self):
        if not len(self.fluendo):
            self.fail("could not find any fluendo products")
        if not len(self.ubuntu):
            self.fail("could not find any ubuntu products")

    def testGuessBestFlavori386(self):
        """ check we have exactly one i386 flavor for all plugins """
        for p in self.products:
            flavs = p.find_compatible_flavors('Linux', 'i386', 'ubuntu', '0.10.0', '0.10.0')
            self.assertEquals(1, len(flavs))

    def testGuessBestFlavorSolaris(self):
        """ check we have exactly one i386 flavor for all plugins """
        for p in self.products:
            flavs = p.find_compatible_flavors('Linux', 'solaris-intel', 'generic', '0.10.0', '0.10.0')
            if p.nick in ['wma-bundle','wmv-bundle','mms','mp3','windows-media-bundle']:
                self.assertEquals(1, len(flavs))
            else:
                self.assertEquals(0, len(flavs))

    def testGuessBestFlavorCurSys(self):
        """ check fluendo mp3 is available on all systems """
        for p in filter(lambda p: p.nick == "mp3", self.products):
            flavs = p.get_flavors_for_current_system()
            self.assertEquals(1, len(flavs))

    def expected_result(self, p, has_it, expected, name):
        """ convenience function to check products have expected provides """
        if p.nick in expected:
            self.assertEquals(True, has_it,
                "Did not find %s as expected in %s" % (name, p.nick))
        else:
            self.assertEquals(False, has_it,
                "Found %s not expected, in %s" % (name, p.nick))

    def testProvidesCodecMP3(self):
        test_caps = 'audio/mpeg, mpegversion=(int)1, layer=(int)3'
        fluendo_matches = ['mpeg-bundle', 'mp3', 'megabundle']
        ubuntu_matches = ['gstreamer0.10-ffmpeg', 'gstreamer0.10-plugins-ugly']
        for p in self.products:
            has_it = p.provides_codec(p.get_provided_decoders(), test_caps)
            self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, "Mp3 Decoder")

    def testProvidesDecoderMP3(self):
        test_caps = 'audio/mpeg, mpegversion=(int)1, layer=(int)3'
        fluendo_matches = ['mpeg-bundle', 'mp3', 'megabundle']
        ubuntu_matches = ['gstreamer0.10-ffmpeg', 'gstreamer0.10-plugins-ugly']
        for p in self.products:
            has_it = p.provides_decoder(test_caps)
            self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, "MP3 Decoder")

    def testProvidesDecoderAC3(self):
        for test_caps in ['audio/x-ac3']:
            fluendo_matches = ['mpeg-bundle', 'ac3-bundle', 'megabundle']
            ubuntu_matches = ['gstreamer0.10-plugins-ugly']
            for p in self.products:
                has_it = p.provides_codec(p.get_provided_decoders(), test_caps)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, "AC3 Decoder")
        # should these caps be in ffmpeg?
        for test_caps in ['audio/x-private1-ac3', 'audio/ac3']:
            fluendo_matches = ['mpeg-bundle', 'ac3-bundle', 'megabundle']
            ubuntu_matches = ['gstreamer0.10-plugins-ugly']
            for p in self.products:
                has_it = p.provides_codec(p.get_provided_decoders(), test_caps)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, "AC3 (private1) Decoder")

    def testProvidesDecoderASF(self):
        for test_proto in ['video/x-ms-asf']:
            fluendo_matches = ['megabundle', 'wmv-bundle', 'wma-bundle', 'windows-media-bundle']
            ubuntu_matches = ['gstreamer0.10-ffmpeg', 'gstreamer0.10-plugins-ugly']
            for p in self.products:
                has_it = p.provides_decoder(test_proto)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, test_proto)
        # should these caps be in ffmpeg?
        for test_proto in ['audio/x-ms-asf']:
            fluendo_matches = ['megabundle', 'wmv-bundle', 'wma-bundle', 'windows-media-bundle']
            ubuntu_matches = []
            for p in self.products:
                has_it = p.provides_decoder(test_proto)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, test_proto)

    def testProvidesDecoderH264(self):
        for test_proto in ['video/x-h264']:
            fluendo_matches = ['megabundle']
            ubuntu_matches = ['gstreamer0.10-ffmpeg']
            for p in self.products:
                has_it = p.provides_decoder(test_proto)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, test_proto)

    def testProvidesProtocolMMS(self):
        for test_proto in ['mms', 'mmsh', 'mmsu', 'mmst']:
            fluendo_matches = ['megabundle', 'mms', 'windows-media-bundle']
            ubuntu_matches = ['gstreamer0.10-plugins-bad']
            for p in self.products:
                has_it = p.provides_urisource(test_proto)
                self.expected_result(p, has_it, fluendo_matches + ubuntu_matches, test_proto)
