# -*- Mode: Python; test-case-name: codeina.test.test_product_list -*-
# vi:si:et:sw=4:sts=4:ts=4

import os
import StringIO

from codeina.test.common import unittest
from codeina import productlistparser
from codeina import productlist

class TestProduct(unittest.TestCase):
    def setUp(self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.i386.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='i386' distro='generic'/>
    <flavor comment=' ' base-req='0.10.0' system='Linux' uri='http://shop.fluendo.com/pub/fluendo-gstreamer-ffmpeg-0.10.1.ppc.tar.bz2' last-modified='2008-02-20' core-req='0.10.0' distro-version='any' arch='ppc' distro='generic'/>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()

    def testProduct(self):
        productlist = self.parser.parse(self.handle, "test")
        self.assertEquals(len(productlist.products), 1)
        p = productlist.products[0]
        self.assertEquals(p.price, 0.0)
        self.assertEquals(p.rank, 0)
        self.assertEquals(p.nick, "gstreamer-ffmpeg")
        self.assertEquals(p.version, '0.10.1')
        self.assertEquals(p.id, 0)
        self.assertEquals(p.logo_uri, 'http://download.fluendo.com/images/windowsmedia.png')
        self.assertEquals(p.name, 'Windows Media Audio Decoder Bundle')
        self.assertEquals(len(p.flavors), 2)
        self.assertEquals(len(p.plugins), 1)
        first_plugin = p.plugins[0]
        self.assertEquals(first_plugin.name, 'ffmpeg')
        self.assertEquals(first_plugin.version, '0.10.1')
        self.assertEquals(first_plugin.description, 'All FFMPEG codecs')
        self.assertEquals(first_plugin.origin, 'http://ffmpeg.sourceforge.net/')
        self.assertEquals(len(first_plugin.elements), 1)
        first_element = first_plugin.elements[0]
        self.assertEquals(first_element.name, 'ffmux_3gp')
        self.assertEquals(len(first_element.decoders), 3)
        self.assertEquals(first_element.decoders[0].caps, 'audio/AMR, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];')
        self.assertEquals(first_element.decoders[1].caps, 'audio/AMR-WB, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];')
        self.assertEquals(first_element.decoders[2].caps, 'audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;')

    def test_product_get_provided_elements(self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'/>
      <element name='foo2'/>
      <element name='foo3'/>
      <element name='foo4'/>
    </plugin>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        self.assertEquals(len(productlist.products), 1)
        p = productlist.products[0]
        self.assertEquals(p.get_provided_elements (), ['ffmux_3gp', 'foo2', 'foo3', 'foo4'])

    def test_product_is_part_product_of (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n4' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p4' description='ping'>
      <element name='foo1'/>
      <element name='foo2'/>
      <element name='foo3'/>
      <element name='foo4'/>
    </plugin>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n1' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p1' description='ping'>
      <element name='foo1'/>
    </plugin>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        self.assertEquals(len(productlist.products), 2)
        p4 = productlist.products[0]
        p1 = productlist.products[1]
        self.assertEquals(p4.get_provided_elements (), ['foo1', 'foo2', 'foo3', 'foo4'])
        self.assertEquals(p1.get_provided_elements (), ['foo1'])
        self.assertEquals(p1.is_part_product_of (p4), True)
        self.assertEquals(p4.is_part_product_of (p1), False)

    def test_products_url_and_id (self):
        xml = """
<products url='http://piem/media/providers.xml' id='mandriva'>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n4' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p4' description='ping'>
      <element name='foo1'/>
      <element name='foo2'/>
      <element name='foo3'/>
      <element name='foo4'/>
    </plugin>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n1' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p1' description='ping'>
      <element name='foo1'/>
    </plugin>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        self.assertEquals (productlist.url, 'http://piem/media/providers.xml')
        self.assertEquals (productlist.id, 'mandriva')

    def testNameTranslated(self):
        LANGUAGE = os.environ.get('LANGUAGE', None)
        os.environ['LANGUAGE'] = 'nl_NL'
        productlist = self.parser.parse(self.handle, "test")
        p = productlist.products[0]
        self.assertEquals(p.name, "Windows Media Audio Decoder Bundel")
        if LANGUAGE is not None:
            os.environ['LANGUAGE'] = LANGUAGE
        else:
            del os.environ['LANGUAGE']

    def testEmptyProduct(self):
        del self.handle, self.parser
        self.handle = StringIO.StringIO("<products></products>")
        self.parser = productlistparser.ProductListParser()
        products = self.parser.parse(self.handle, "test")

    def testDuplicateNicks(self):
        del self.handle, self.parser
        xml = """
<products>
  <product id="42"
           nick="wma-dec-bundle"
           rank="75"
           price="EUR 7.00"
           version="0.10.0"
           logo-uri="http://download.fluendo.com/images/windowsmedia.png">
      <name>Windows Media Audio Decoder Bundle</name>
      <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
      <summary>Everything you need to listen to ASF/WMA audio streams and files</summary>
      <flavor system="Linux" arch="i386" distro="generic" distro-version="any"
              core-req="0.10.0" base-req="0.10.0"
              last-modified="2007-03-26"
              uri="http://download.fluendo.com/plugins/fluendo-ac3-bundle-1.i386.tar.bz2"
              comment="Tested on FC3, FC4, FC5, and Ubuntu 6.10 (edgy)"
              />
  </product>
  <product id="43"
           nick="wma-dec-bundle"
           rank="75"
           price="EUR 7.00"
           version="0.10.0"
           logo-uri="http://download.fluendo.com/images/windowsmedia.png">
      <name>Windows Media Audio Decoder Bundle</name>
      <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
      <summary>Everything you need to listen to ASF/WMA audio streams and files</summary>
      <flavor system="Linux" arch="i386" distro="generic" distro-version="any"
              core-req="0.10.0" base-req="0.10.0"
              last-modified="2007-03-26"
              uri="http://download.fluendo.com/plugins/fluendo-ac3-bundle-1.i386.tar.bz2"
              comment="Tested on FC3, FC4, FC5, and Ubuntu 6.10 (edgy)"
              />
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        products = self.parser.parse(self.handle, "test")

    def test_product_get_installed_plugins (self):
        xml = """
<products url='http://piem/media/providers.xml' id='mandriva'>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n4' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p4' description='ping'>
      <element name='foo1'/>
    </plugin>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n1' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p1' description='ping'>
      <element name='foo1'/>
    </plugin>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        product = productlist.products[0]
        self.assertEquals (product.get_installed_plugins (), [])
        product.plugins[0].cache_is_installed = True
        self.assertEquals (product.get_installed_plugins (), [product.plugins[0]])

    def test_product_get_installed_plugins_partially_installed (self):
        xml = """
<products url='http://piem/media/providers.xml' id='mandriva'>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n4' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p4' description='ping'>
      <element name='foo1'/>
    </plugin>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='p1' description='ping'>
      <element name='foo1'/>
    </plugin>
  </product>
</products>
"""
        self.handle = StringIO.StringIO(xml)
        self.parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(self.handle, "test")
        product = productlist.products[0]
        product.flush_cache ()
        plugin1 = product.plugins[0]
        plugin2 = product.plugins[1]
        plugin1.cache_is_installed = False
        plugin2.cache_is_installed = True
        plugin1.cache_is_newer = False
        plugin2.cache_is_newer = False
        self.assertEquals ( product.get_installed_plugins (), [plugin2])
        self.assertEquals ( product.is_partially_installed (), True)
        self.assertEquals ( product.is_fully_installed (), False)
        self.assertEquals ( product.updates_available (), False)
        product.flush_cache ()
        plugin1.cache_is_installed = False
        plugin2.cache_is_installed = True
        plugin1.cache_is_newer = False
        plugin2.cache_is_newer = True
        self.assertEquals ( product.updates_available (), True)

    def test_element_urisources(self):
        xml = """
<products url='http://piem/media/providers.xml' id='mandriva'>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='n4' version='0.10.1' id='0'>
    <plugin origin='http://gstreamer.net/' version='0.10.5' name='mms' description='Microsoft Multi Media Server streaming protocol support'>
      <element name='mmssrc'>
        <urisource>mms</urisource>
      </element>
      <element name='mmssrc'>
        <urisource>mmsh</urisource>
      </element>
      <element name='mmssrc'>
        <urisource>mmst</urisource>
      </element>
      <element name='mmssrc'>
        <urisource>mmsu</urisource>
      </element>
    </plugin>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = self.parser.parse(handle, "test")
        product = productlist.products[0]
        product.flush_cache ()
        plugin1 = product.plugins[0]
        self.assertEqual (plugin1.element_count (), 4)
        self.assertEqual (plugin1.elements[0].name, 'mmssrc')
        self.assertEqual (plugin1.elements[1].name, 'mmssrc')
        self.assertEqual (plugin1.elements[2].name, 'mmssrc')
        self.assertEqual (plugin1.elements[3].name, 'mmssrc')
        self.assertEqual (plugin1.elements[0].decoder_count (), 0)
        self.assertEqual (plugin1.elements[1].decoder_count (), 0)
        self.assertEqual (plugin1.elements[2].decoder_count (), 0)
        self.assertEqual (plugin1.elements[3].decoder_count (), 0)
        self.assertEqual (plugin1.elements[0].urisources_count (), 1)
        self.assertEqual (plugin1.elements[1].urisources_count (), 1)
        self.assertEqual (plugin1.elements[2].urisources_count (), 1)
        self.assertEqual (plugin1.elements[3].urisources_count (), 1)
        self.assertEqual (plugin1.elements[3].check_legal (), False)

