# -*- Mode: Python; test-case-name: codeina.test.test_httpdownload -*-
# vi:si:et:sw=4:sts=4:ts=4

import os

from codeina.test.common import unittest
from codeina.requiredplugin import RequiredPlugin 

mpegdecoder_desc = 'gstreamer|0.10'\
                '|totem'\
                '|MP3 Decoder'\
                '|decoder-audio/mpeg, mpegversion=(int)1, layer=(int)3'

urisource_mms_desc = 'gstreamer|0.10'\
                '|totem'\
                '|Microsoft Media Server (MMS) protocol source'\
                '|urisource-mms'

class TestRequiredPlugin(unittest.TestCase):

    def test_is_mp3_decoder_with_empty_desc(self):
        req = RequiredPlugin()
        self.assertEqual(False, req.valid)
        self.assertEqual(False, req.is_mp3_decoder())

    def test_is_mp3_decoder_with_correct_desc(self):
        req = RequiredPlugin(mpegdecoder_desc)
        self.assertEqual(True, req.valid)
        self.assertEqual(True, req.is_mp3_decoder())
        
    def test_is_mp3_decoder_with_incorrect_desc(self):
        req = RequiredPlugin(urisource_mms_desc)
        self.assertEqual(True, req.valid)
        self.assertEqual(False, req.is_mp3_decoder())

    def test_is_mp3_decoder_with_no_mpegversion(self):
        req = RequiredPlugin(mpegdecoder_desc.replace('mpegversion','foobarversion'))
        self.assertEqual(True, req.valid)
        self.assertEqual(False, req.is_mp3_decoder())

    def test_is_mp3_decoder_with_wrong_mpegversion(self):
        req = RequiredPlugin(mpegdecoder_desc.replace('mpegversion=(int)1','mpegversion=(int)2'))
        self.assertEqual(True, req.valid)
        self.assertEqual(False, req.is_mp3_decoder())

    def test_is_mp3_decoder_with_wrong_mimetype(self):
        req = RequiredPlugin(mpegdecoder_desc.replace('audio/mpeg','audio/foobar'))
        self.assertEqual(True, req.valid)
        self.assertEqual(False, req.is_mp3_decoder())

    def test_required_plugin_repr(self):
        req = RequiredPlugin('gstreamer|0.10|totem|Microsoft Media Server (MMS)'\
                          ' protocol source|urisource-mms')
        self.assertEqual(True, req.valid)
        self.assertEqual("<RequiredPlugin for urisource-mms> ", str(req) )
        
    def test_required_plugin_is_decoder(self):
        """ check all possible req_type, including one invalid one """
        req = RequiredPlugin('gstreamer|0.10|totem|some description|urisource-mms')
        for real_type in req.valid_req_types + ["invalid"]:
            req.req_type = real_type
            for test_type in req.valid_req_types + ["invalid"]:
                if test_type == real_type and test_type != "invalid":
                    self.assertEqual(True, req.is_req_type(test_type))
                else:
                    self.assertEqual(False, req.is_req_type(test_type))

                if hasattr(req,"is_"+test_type):
                    exec("a = req.is_"+test_type+"()")
                    #print "real_type ("+real_type+") ?= req_type("+test_type+")",
                    #print real_type == test_type
                    #print "req.is_"+test_type+"() is ", a
                    self.assertEqual(real_type == test_type,  a)
