# -*- Mode: Python; test-case-name: codeina.test.test_product_list -*-
# vi:si:et:sw=4:sts=4:ts=4

from codeina.test.common import unittest
from codeina import productlistparser
from codeina import productlist
import StringIO

class TestRestrictedProducts (unittest.TestCase):

    def test_element_type_legal (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        e0 = productlist.products[0].plugins[0].elements[0].decoders[0]
        e1 = productlist.products[0].plugins[0].elements[0].decoders[1]
        self.assertEqual (e0.check_legal (), True)
        self.assertEqual (e1.check_legal (), False)

    def test_element_legal (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        e0 = productlist.products[0].plugins[0].elements[0]
        e1 = productlist.products[0].plugins[0].elements[1]
        self.assertEqual (e0.check_legal (), False)
        self.assertEqual (e1.check_legal (), True)

    def test_plugin_caps_legal (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='exotic1' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='exotic2' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        e0 = productlist.products[0].plugins[0]
        e1 = productlist.products[0].plugins[1]
        self.assertEqual (e0.check_legal (), False)
        self.assertEqual (e1.check_legal (), True)

    def test_plugin_name_legal (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='vorbisdec' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='exotic1' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
    <name>Windows Media Audio Decoder Bundle</name>
    <name xml:lang="nl">Windows Media Audio Decoder Bundel</name>
    <summary>Missing Summary</summary>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        e0 = productlist.products[0].plugins[0]
        e1 = productlist.products[0].plugins[1]
        self.assertEqual (e0.check_legal (), True)
        self.assertEqual (e1.check_legal (), False)

    def test_product_legal (self):
        xml = """
<products>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer1-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='ffmpeg' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
  </product>
  <product logo-uri='http://download.fluendo.com/images/windowsmedia.png' price='0.00' rank='0' nick='gstreamer-ffmpeg' version='0.10.1' id='0'>
    <plugin origin='http://ffmpeg.sourceforge.net/' version='0.10.1' name='vorbisdec' description='All FFMPEG codecs'>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
        <decoder>audio/mpeg, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ], mpegversion=(int)4;</decoder>
      </element>
      <element name='ffmux_3gp'>
        <decoder>audio/x-vorbis, rate=(int)[ 8000, 96000 ], channels=(int)[ 1, 2 ];</decoder>
      </element>
    </plugin>
  </product>
</products>
"""
        handle = StringIO.StringIO(xml)
        parser = productlistparser.ProductListParser()
        productlist = parser.parse(handle, "test")
        e0 = productlist.products[0]
        e1 = productlist.products[1]
        self.assertEqual (e0.check_legal (), False)
        self.assertEqual (e1.check_legal (), True)
